@@ -86,6 +86,11 @@ extern "C" {
  */
 #define ZT_MIN_PHYSMTU 1400
 
+/**
+ * Maximum physical interface name length. This number is gigantic because of Windows.
+ */
+#define ZT_MAX_PHYSIFNAME 256
+
 /**
  * Default UDP payload size (physical path MTU) not including UDP and IP overhead
  *
@@ -1317,35 +1322,20 @@ typedef struct
 	 */
 	float packetErrorRatio;
 
-	/**
-	 * Mean throughput
-	 */
-	uint64_t throughputMean;
-
-	/**
-	 * Maximum observed throughput
-	 */
-	float throughputMax;
-
-	/**
-	 * Throughput variance
-	 */
-	float throughputVariance;
-
 	/**
 	 * Address scope
 	 */
 	uint8_t scope;
 
 	/**
-	 * Percentage of traffic allocated to this path
+	 * Relative quality value
 	 */
-	float allocation;
+	float relativeQuality;
 
 	/**
-	 * Name of physical interface (for monitoring)
+	 * Name of physical interface this path resides on
 	 */
-	char ifname[32];
+	char ifname[ZT_MAX_PHYSIFNAME];
 
 	uint64_t localSocket;
 
@@ -1354,6 +1344,21 @@ typedef struct
 	 */
 	int expired;
 
+	/**
+	 * Whether this path is currently included in the bond
+	 */
+	uint8_t bonded;
+
+	/**
+	 * Whether this path is currently eligible to be used in a bond
+	 */
+	uint8_t eligible;
+
+	/**
+	 * The capacity of this link (as given to bonding layer)
+	 */
+	uint32_t linkSpeed;
+
 	/**
 	 * Is path preferred?
 	 */
